@@ -21,6 +21,8 @@
  */
 
 #include <math.h>
+#include <float.h>
+
 #include <pds/pdsra.h>
 #include <pds/pdsrv.h>
 #include <pds/pdssn.h>
@@ -35,8 +37,7 @@
  *  <li>El máximo margen de error RMS aceptado en la salida de la estructura 
     PdsNeuronML es PDS_DEFAULT_MAX_RMS_ERROR.</li>
  *  <li>El valor de de Alpha se inicia con PDS_DEFAULT_ALPHA.</li>
- *  <li>El valor U se inicia con 1.0.</li>
- *  <li>El valor Sigma se inicia con 1.0.</li>
+ *  <li>El valor U y W son iniciados aleatoriamente entre -1.0 1.0.</li>
  *  <li>El valor func(x) se inicia con tanh(x/2).</li>
  *  </ul> 
  *  
@@ -45,19 +46,60 @@
  *  \ingroup PdsNeuronMLGroup
  */
 PdsNeuronML *pds_neuronml_new(const PdsVector *L)
+{
+	PdsNeuronML *NeuronML=NULL;
+	PdsNnNatural i;
+
+	if(L==NULL)		return NULL;
+	if(L->Nel<2)	return NULL;
+	
+    PdsNnNatural *Larray=(PdsNnNatural *)calloc(L->Nel,sizeof(PdsNnNatural));
+
+    for(i=0;i<L->Nel;i++)
+    {
+        Larray[i]=(PdsNnNatural)fabs(L->V[i]);
+    }
+
+	// Reservo memoria de la estructura.
+	NeuronML=pds_neuronml_new_from_array(Larray,L->Nel);
+	if(NeuronML==NULL)	return NULL;
+
+	return NeuronML;
+}
+
+/** \fn PdsNeuronML *pds_neuronml_new_from_array(const PdsNnNatural *L,PdsNnNatural Nel)
+ *  \brief Crea una estructura de tipo PdsNeuronML.
+ *
+ *  <ul>
+ *  <li>Inicia con cero todos los Pesos. </li>
+ *  <li>El máximo margen de error RMS aceptado en la salida de la estructura 
+    PdsNeuronML es PDS_DEFAULT_MAX_RMS_ERROR.</li>
+ *  <li>El valor de de Alpha se inicia con PDS_DEFAULT_ALPHA.</li>
+ *  <li>El valor U y W son iniciados aleatoriamente entre -1.0 1.0.</li>
+ *  <li>El valor func(x) se inicia con tanh(x/2).</li>
+ *  </ul> 
+ *  
+ *  \param[in] L Es un arreglo con el numero de elementos por capas.
+ *  \param[in] Nel Número de elementos del arreglo.
+ *  \return Un puntero al vector de tipo PdsNeuronML.
+ *  \ingroup PdsNeuronMLGroup
+ */
+PdsNeuronML *pds_neuronml_new_from_array(const PdsNnNatural *L,PdsNnNatural Nel)
 {
 	PdsNeuronML *NeuronML=NULL;
 	PdsNnNatural i,j;
 
 	if(L==NULL)		return NULL;
-	if(L->Nel<2)		return NULL;
+	if(Nel<2)		return NULL;
 	
+    for(i=0;i<Nel;i++)  if(L[i]==0) return NULL;
+
 	// Reservo memoria de la estructura.
 	NeuronML=(PdsNeuronML *)calloc(1,sizeof(PdsNeuronML));
 	if(NeuronML==NULL)	return NULL;
 
 	// Reservo vector de capas.
-	NeuronML->L=(PdsNnNatural*)calloc(L->Nel,sizeof(PdsNnNatural));
+	NeuronML->L=(PdsNnNatural*)calloc(Nel,sizeof(PdsNnNatural));
 
 	if(NeuronML->L==NULL)	
 	{
@@ -65,22 +107,13 @@ PdsNeuronML *pds_neuronml_new(const PdsVector *L)
 		return NULL;
 	}
 
-	for(i=0;i<L->Nel;i++)
+	for(i=0;i<Nel;i++)
 	{
-		if(L->V[i]>0)
-		{
-			if(L->V[i]>1)	NeuronML->L[i]=(PdsNnNatural)L->V[i];
-			else		NeuronML->L[i]=1;
-		}
-		else
-		{
-			if(L->V[i]<-1)	NeuronML->L[i]=(PdsNnNatural)(-L->V[i]);
-			else		NeuronML->L[i]=1;
-		}
+		NeuronML->L[i]=(PdsNnNatural)L[i];
 	}
 
 	// Numero de capas.
-	NeuronML->N=L->Nel-1;
+	NeuronML->N=Nel-1;
 
 	// Creo vector de neuronas de entrada.
 	NeuronML->LayerInput=pds_nivector_new(NeuronML->L[0]);
@@ -93,7 +126,7 @@ PdsNeuronML *pds_neuronml_new(const PdsVector *L)
 
 
 
-	NeuronML->Layer=(PdsNVector **)calloc(L->Nel-1,sizeof(PdsNVector *));
+	NeuronML->Layer=(PdsNVector **)calloc(Nel-1,sizeof(PdsNVector *));
 	if(NeuronML->Layer==NULL) 
 	{
 		free(NeuronML->L);
@@ -129,11 +162,12 @@ PdsNeuronML *pds_neuronml_new(const PdsVector *L)
 	NeuronML->Iter=0;
 	NeuronML->ShowPrintf=0;
 
+    pds_neuronml_init_u_rand(NeuronML,-1,1);
+	pds_neuronml_init_weight_rand(NeuronML,-1,1);
 
 	return NeuronML;
 }
 
-
 /** \fn PdsNeuronML *pds_neuronml_new_load_data(const char *mlayer_filename)
  *  \brief Crea una estructura de tipo PdsNeuronML e inicia todos los elementos.
  *  desde un archivo de texto.
@@ -148,7 +182,6 @@ PdsNeuronML *pds_neuronml_new(const PdsVector *L)
  *  <li>ShowPrintf, Si la estructura muestra información adicional.</li>
  *  <li>L, Vector de N+1 elementos con el número de neuronas por capa.</li>
  *  <li>Vector de pesos de neurona.</li>
- *  <li>Valor de U y Valor de Sigma</li>
  *  <li>...</li>
  *  </ul> 
  *  
@@ -160,135 +193,15 @@ PdsNeuronML *pds_neuronml_new_load_data(const char *mlayer_filename)
 {
 	PdsNeuronML *NeuronML=NULL;
 	FILE *fd=NULL;
-	PdsNnNatural i,j;
-	PdsNnReal x;
-	int tmp;
 
 	if(mlayer_filename==NULL)	return NULL;
 
 	fd=fopen(mlayer_filename,"r");
 	if(fd==NULL)	return NULL;
 
-	// Reservo memoria de la estructura.
-	NeuronML=(PdsNeuronML *)calloc(1,sizeof(PdsNeuronML));
-	if(NeuronML==NULL)	return NULL;
+    NeuronML=pds_neuronml_new_head_fscanf(fd);
 
-	if(feof(fd)==0)	
-	{
-		tmp=fscanf(fd,"%e",&x);
-		if(tmp==0)	{free(NeuronML);	return NULL;}
-		NeuronML->N=(PdsNnNatural)x;
-	}
-	else		{free(NeuronML);	return NULL;}
-
-	if(feof(fd)==0)	
-	{
-		tmp=fscanf(fd,"%e",&(NeuronML->MaxError));
-		if(tmp==0)	{free(NeuronML);	return NULL;}
-	}
-	else		{free(NeuronML);	return NULL;}
-
-	if(feof(fd)==0)	
-	{
-		tmp=fscanf(fd,"%e",&(NeuronML->Alpha));
-		if(tmp==0)	{free(NeuronML);	return NULL;}
-	}
-	else		{free(NeuronML);	return NULL;}
-
-	if(feof(fd)==0)	
-	{
-		tmp=fscanf(fd,"%e",&x);
-		if(tmp==0)	{free(NeuronML);	return NULL;}
-		NeuronML->MaxIter=(PdsNnNatural)x;
-	}
-	else		{free(NeuronML);	return NULL;}
-
-	if(feof(fd)==0)	
-	{
-		tmp=fscanf(fd,"%e",&x);
-		if(tmp==0)	{free(NeuronML);	return NULL;}
-		NeuronML->Iter=(PdsNnNatural)x;
-	}
-	else		{free(NeuronML);	return NULL;}
-
-	if(feof(fd)==0)	
-	{
-		tmp=fscanf(fd,"%e",&x);
-		if(tmp==0)	{free(NeuronML);	return NULL;}
-		NeuronML->ShowPrintf=(PdsNnBool)x;
-	}
-	else		{free(NeuronML);	return NULL;}
-
-	// Reservo vector de capas.
-	NeuronML->L=(PdsNnNatural*)calloc(NeuronML->N+1,sizeof(PdsNnNatural));
-
-	if(NeuronML->L==NULL)	{free(NeuronML);	return NULL;}
-
-
-	for(i=0;i<=NeuronML->N;i++)
-	{
-		x=0;
-		if(feof(fd)==0)	
-		{
-			tmp=fscanf(fd,"%e",&x);
-			if(tmp==0)	
-			{free(NeuronML->L);	free(NeuronML);	return NULL;}
-		}
-		else	{free(NeuronML->L);	free(NeuronML);	return NULL;}
-
-		if(x>0)
-		{
-			if(x>1)		NeuronML->L[i]=(PdsNnNatural)x;
-			else		NeuronML->L[i]=1;
-		}
-		else
-		{
-			if(x<-1)	NeuronML->L[i]=(PdsNnNatural)(-x);
-			else		NeuronML->L[i]=1;
-		}
-	}
-
-	// Creo vector de neuronas de entrada.
-	NeuronML->LayerInput=pds_nivector_new(NeuronML->L[0]);
-	if(NeuronML->LayerInput==NULL)	
-	{
-		free(NeuronML->L);
-		free(NeuronML);
-		return NULL;
-	}
-
-
-	// Creo vector de neuronas de capas oculta y de salida.
-	NeuronML->Layer=(PdsNVector **)calloc(NeuronML->N,sizeof(PdsNVector *));
-	if(NeuronML->Layer==NULL) 
-	{
-		free(NeuronML->L);
-		pds_nivector_free(NeuronML->LayerInput);
-		free(NeuronML);
-		return NULL;
-	}
-
-	for(i=0;i<NeuronML->N;i++)
-	{	
-		NeuronML->Layer[i]=pds_nvector_new(NeuronML->L[i+1],NeuronML->L[i]);
-		if(NeuronML->Layer[i]==NULL)
-		{
-			for(j=0;j<i;j++)	pds_nvector_free(NeuronML->Layer[i]);
-			free(NeuronML->L);
-			pds_nivector_free(NeuronML->LayerInput);
-			free(NeuronML);
-			return NULL;
-		}
-		
-	}
-
-	pds_nvector_connect_input_with_nivector(NeuronML->Layer[0],NeuronML->LayerInput);
-	for(i=1;i<NeuronML->N;i++)
-	{
-		pds_nvector_connect_input(NeuronML->Layer[i],NeuronML->Layer[i-1]);
-	}
-
-	pds_neuronml_fscanf(NeuronML,fd);
+	pds_neuronml_body_fscanf(NeuronML,fd);
 	
 	fclose(fd);
 
@@ -309,7 +222,6 @@ PdsNeuronML *pds_neuronml_new_load_data(const char *mlayer_filename)
  *  <li>ShowPrintf, Si la estructura muestra información adicional.</li>
  *  <li>L, Vector de N+1 elementos con el número de neuronas por capa.</li>
  *  <li>Vector de pesos de neurona.</li>
- *  <li>Valor de U y Valor de Sigma</li>
  *  <li>...</li>
  *  </ul> 
  *  
@@ -321,7 +233,7 @@ PdsNeuronML *pds_neuronml_new_load_data(const char *mlayer_filename)
 int pds_neuronml_save_data(const PdsNeuronML *NeuronML, const char *mlayer_filename)
 {
 	FILE *fd=NULL;
-	PdsNnNatural i;
+	//PdsNnNatural i;
 
 	if(NeuronML==NULL)	return FALSE;
 	if(mlayer_filename==NULL)	return FALSE;
@@ -329,25 +241,9 @@ int pds_neuronml_save_data(const PdsNeuronML *NeuronML, const char *mlayer_filen
 	fd=fopen(mlayer_filename,"w");
 	if(fd==NULL)	return FALSE;
 
-	fprintf(fd,"%u\t",NeuronML->N);
+    pds_neuronml_head_fprintf(NeuronML,fd);
 
-	fprintf(fd,"%e\t",NeuronML->MaxError);
-
-	fprintf(fd,"%e\t",NeuronML->Alpha);
-
-	fprintf(fd,"%u\t",NeuronML->MaxIter);
-
-	fprintf(fd,"%u\t",NeuronML->Iter);
-
-	fprintf(fd,"%u\n",NeuronML->ShowPrintf);
-
-	for(i=0;i<=NeuronML->N;i++)
-	{	
-		if(i!=NeuronML->N)	fprintf(fd,"%u\t",NeuronML->L[i]);
-		else			    fprintf(fd,"%u\n",NeuronML->L[i]);
-	}
-
-	pds_neuronml_fprintf(NeuronML,fd);
+	pds_neuronml_body_fprintf(NeuronML,fd);
 
 	fclose(fd);
 
@@ -355,6 +251,78 @@ int pds_neuronml_save_data(const PdsNeuronML *NeuronML, const char *mlayer_filen
 }
 
 
+/** \fn int pds_neuronml_get_number_of_input_neurons(PdsNeuronML *NeuronML,PdsNnNatural *Number)
+ *  \brief Retorna o numero de neuronas de entrada.
+ *  \param[in] NeuronML Red neuronal Multi capa a cosultar.
+ *  \param[out] Number Número de neuronas de entrada.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_number_of_input_neurons(PdsNeuronML *NeuronML,PdsNnNatural *Number)
+{
+	*Number=0;
+
+	if(NeuronML==NULL)	return FALSE;
+
+	*Number=NeuronML->L[0];
+
+	return TRUE;
+}
+
+/** \fn int pds_neuronml_get_number_of_output_neurons(PdsNeuronML *NeuronML,PdsNnNatural *Number)
+ *  \brief Retorna o numero de neuronas de salida.
+ *  \param[in] NeuronML Red neuronal Multi capa a cosultar.
+ *  \param[out] Number Número de neuronas de salida.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_number_of_output_neurons(PdsNeuronML *NeuronML,PdsNnNatural *Number)
+{
+	*Number=0;
+
+	if(NeuronML==NULL)	return FALSE;
+
+	*Number=NeuronML->L[NeuronML->N];
+
+	return TRUE;
+}
+
+/** \fn int pds_neuronml_get_maximum_output(PdsNeuronML *NeuronML,PdsNnReal *Max)
+ *  \brief Retorna o valor maximo de las neuronas de salida
+ *  \param[in] NeuronML Red neuronal Multi capa a cosultar.
+ *  \param[out] Max Valor máximo de las neuronas de salida.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_maximum_output(PdsNeuronML *NeuronML,PdsNnReal *Max)
+{
+	*Max=0;
+
+	if(NeuronML==NULL)	return FALSE;
+
+	*Max=NeuronML->Layer[NeuronML->N-1]->V[0]->func(FLT_MAX);
+
+	return TRUE;
+}
+
+/** \fn int pds_neuronml_get_minimum_output(PdsNeuronML *NeuronML,PdsNnReal *Min)
+ *  \brief Retorna o valor maximo de las neuronas de salida
+ *  \param[in] NeuronML Red neuronal Multi capa a cosultar.
+ *  \param[out] Min Valor mínimo de las neuronas de salida.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_minimum_output(PdsNeuronML *NeuronML,PdsNnReal *Min)
+{
+	*Min=0;
+
+	if(NeuronML==NULL)	return FALSE;
+
+	*Min=NeuronML->Layer[NeuronML->N-1]->V[0]->func(-FLT_MAX);
+
+	return TRUE;
+}
+
 /** \fn int pds_neuronml_set_max_error(PdsNeuronML *NeuronML,PdsNnReal MaxError)
  *  \brief Coloca el maximo margen error RMS aceptado en la salida de la estructura
  *  PdsNeuronML.
@@ -479,6 +447,30 @@ int pds_neuronml_init_weight_uniform(PdsNeuronML *NeuronML,PdsUniform *RV)
 }
 
 
+/** \fn int pds_neuronml_init_weight_rand(PdsNeuronML *NeuronML,PdsSnReal min, PdsSnReal max)
+ *  \brief Inicializa todos los pesos entre los valores min y max, 
+ *  usando internamente la funcion rand.
+ *  \param[in,out] NeuronML Red neuronal Multi capa a trabajar.
+ *  \param[in] min Valor mínimo.
+ *  \param[in] max Valor máximo.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_init_weight_rand(PdsNeuronML *NeuronML,PdsSnReal min, PdsSnReal max)
+{
+	PdsNnNatural i;
+	
+	if(NeuronML==NULL)	return FALSE;
+
+	for(i=0;i<NeuronML->N;i++)
+	{
+		pds_nvector_init_weight_rand(NeuronML->Layer[i],min,max);
+	}
+
+	return TRUE;
+}
+
+
 /** \fn int pds_neuronml_init_u_uniform(PdsNeuronML *NeuronML,PdsUniform *RV)
  *  \brief Inicializa todos los umbrales U usando una variable aleatoria uniforme.
  *  \param[in,out] NeuronML Red neuronal Multi capa a trabajar.
@@ -501,15 +493,15 @@ int pds_neuronml_init_u_uniform(PdsNeuronML *NeuronML,PdsUniform *RV)
 	return TRUE;
 }
 
-
-/** \fn int pds_neuronml_set_sigma(PdsNeuronML *NeuronML,PdsNnReal Sigma)
- *  \brief Coloca el valor de Sigma en todas las neuronas
+/** \fn int pds_neuronml_init_u_rand(PdsNeuronML *NeuronML,PdsSnReal min, PdsSnReal max)
+ *  \brief Inicializa todos los umbrales U de forma aleatoria entre min y max.
  *  \param[in,out] NeuronML Red neuronal Multi capa a trabajar.
- *  \param[in] Sigma Valor de sigma en la función de activación de las neuronas.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL, RV==NULL). 
+ *  \param[in] min Valor mínimo.
+ *  \param[in] max Valor máximo.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_set_sigma(PdsNeuronML *NeuronML,PdsNnReal Sigma)
+int pds_neuronml_init_u_rand(PdsNeuronML *NeuronML,PdsSnReal min, PdsSnReal max)
 {
 	PdsNnNatural i;
 	
@@ -517,7 +509,7 @@ int pds_neuronml_set_sigma(PdsNeuronML *NeuronML,PdsNnReal Sigma)
 
 	for(i=0;i<NeuronML->N;i++)
 	{
-		pds_nvector_set_sigma(NeuronML->Layer[i],Sigma);
+		pds_nvector_init_u_rand(NeuronML->Layer[i],min,max);
 	}
 
 	return TRUE;
@@ -527,7 +519,7 @@ int pds_neuronml_set_sigma(PdsNeuronML *NeuronML,PdsNnReal Sigma)
 /** \fn int pds_neuronml_iterate(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector *Output)
  *  \brief Itera la red neuronal multicapa NeuronML.
  *  \f[ \theta_j=\sum_{k=0}^{N_{d}-1}{W_{kj}X_k} - U_j \f]
- *  \f[ y_j \leftarrow func\left(\frac{\theta_j}{\sigma_j}\right) \f]
+ *  \f[ y_j \leftarrow func\left(\theta_j\right) \f]
  *  \param[in,out] NeuronML Red neuronal Multi capa a trabajar.
  *  \param[in] Input Datos de entrada de la red neuronal multicapa.
  *  \param[out] Output Datos de salida de la red neuronal multicapa.
@@ -545,7 +537,7 @@ int pds_neuronml_iterate(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector
 	if(NeuronML->LayerInput->Nel!=Input->Nel)	return FALSE;
 	if(NeuronML->Layer[N-1]->Nel!=Output->Nel)	return FALSE;
 
-	pds_nivector_set_input(NeuronML->LayerInput,Input);
+	pds_nivector_set_input_from_array(NeuronML->LayerInput,Input->V,Input->Nel);
 
 	for(i=0;i<N;i++)	pds_nvector_iterate(NeuronML->Layer[i]);
 
@@ -555,6 +547,33 @@ int pds_neuronml_iterate(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector
 }
 
 
+/** \fn PdsNnBool pds_neuronml_iterate_results_printf(const PdsNeuronML *MLayer,const char* Type)
+ *  \brief Muestra los resultados de la iteración de la red neuronal multicapa NeuronML.
+ *
+ *  \param[in] MLayer Red neuronal Multi capa a trabajar.
+ *  \param[in] Type Etiqueda da iteracion.
+ *  \return El valor medio de las salidas de la red neuronal multicapa NeuronML.
+ *  \ingroup PdsNeuronMLGroup
+ */
+PdsNnBool pds_neuronml_iterate_results_printf(const PdsNeuronML *MLayer,const char* Type)
+{
+    if(MLayer==NULL)    return FALSE;
+    
+    printf("[%s]\t",Type);
+    
+    PdsVector *Y=NULL;
+    Y=pds_vector_new(MLayer->L[MLayer->N]);
+    pds_neuronml_get_output(MLayer,Y);
+    printf("Y= {\t");
+    for (int i=0;i<Y->Nel;i++)
+    {
+        printf("%5.3f\t",Y->V[i]);
+    }
+    printf("}\n");
+    pds_vector_destroy(&Y);
+    
+    return TRUE;
+}
 
 /** \fn int pds_neuronml_training(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector *Output)
  *  \brief Entrena la red neuronal multicapa NeuronML. Usa el valor de Output como 
@@ -562,9 +581,10 @@ int pds_neuronml_iterate(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector
  *
  *  Bucle de retropropagación e iteración :
  *  <ul>
- *  <li>pds_nvector_evaluate_error()</li>
- *  <li>pds_nvector_backpropagate_error() : \f$ e_k \leftarrow e_k + \frac{func'\left(\frac{\theta_j}{\sigma_j}\right)}{\sigma_j} W_{kj} e_j \f$ </li>
- *  <li>pds_nvector_update_weight() : \f$ W_{kj} = W_{kj} + \alpha e_j \frac{func'\left(\frac{\theta_j}{\sigma_j}\right)}{\sigma_j} \frac{X_{k}}{X^TX} \f$</li>
+ *  <li>pds_nvector_evaluate_diff_error()</li>
+ *  <li>pds_nvector_backpropagate_error() : \f$ e_k \leftarrow e_k + \frac{\alpha}{N_j} func'\left(\theta_j\right) W_{kj} e_j \f$ </li>
+ *  <li>pds_nvector_update_u_and_weight() : \f$ W_{kj} = W_{kj} + \alpha e_j func'\left(\theta_j\right) \frac{X_{k}}{X^TX} \f$</li>
+ *  <li>pds_nvector_update_u_and_weight() : \f$ U_{k} = U_{k} - \alpha e_j func'\left(\theta_j\right) \frac{1}{X^TX} \f$</li>
  *  <li>pds_neuronml_iterate()</li>
  *  </ul>
  *  El bucle es repetido hasta que el error cuadratico medio en la salida sea menor
@@ -581,7 +601,7 @@ int pds_neuronml_training(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector
 {
 	PdsNnNatural N,i,j;
 	PdsNnReal e;
-	PdsSnReal XtX;
+	PdsSnReal XtX=0.0;
 
 	if(NeuronML==NULL)	return FALSE;
 
@@ -589,11 +609,11 @@ int pds_neuronml_training(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector
 	if(NeuronML->LayerInput->Nel!=Input->Nel)	return FALSE;
 	if(NeuronML->Layer[N-1]->Nel!=Output->Nel)	return FALSE;
 
-	pds_nivector_set_input(NeuronML->LayerInput,Input);
+	pds_nivector_set_input_from_array(NeuronML->LayerInput,Input->V,Input->Nel);
 
 	for(i=0;i<N;i++)	pds_nvector_iterate(NeuronML->Layer[i]);
 
-	pds_nvector_evaluate_error(NeuronML->Layer[N-1],Output);
+	pds_nvector_evaluate_diff_error(NeuronML->Layer[N-1],Output);
 
 	pds_nvector_get_rms_error(NeuronML->Layer[N-1],&e);
 
@@ -602,19 +622,19 @@ int pds_neuronml_training(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector
 	while(e>(NeuronML->MaxError))
 	{
 		for(i=0;i<N;i++)
-		pds_nvector_backpropagate_error(NeuronML->Layer[N-1-i]);
+		pds_nvector_backpropagate_error(NeuronML->Layer[N-1-i],NeuronML->Alpha/NeuronML->L[N-1-i]);
 
 		for(i=0;i<N;i++)
 		{
 			pds_neuron_get_xtx(NeuronML->Layer[i]->V[0],&XtX);
-			pds_nvector_update_weight(NeuronML->Layer[i],(NeuronML->Alpha)/XtX);
+			pds_nvector_update_u_and_weight(NeuronML->Layer[i],(NeuronML->Alpha)/(XtX+1));
 		}
 
 		pds_nivector_iterate(NeuronML->LayerInput);
 		for(i=0;i<N;i++)	
 		pds_nvector_iterate(NeuronML->Layer[i]);
 
-		pds_nvector_evaluate_error(NeuronML->Layer[N-1],Output);
+		pds_nvector_evaluate_diff_error(NeuronML->Layer[N-1],Output);
 
 		pds_nvector_get_rms_error(NeuronML->Layer[N-1],&e);
 
@@ -629,32 +649,58 @@ int pds_neuronml_training(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector
 	return TRUE;
 }
 
-
-/** \fn int pds_neuronml_get_hard_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
- *  \brief Evalúa una votación con desición abrupta de los valores de salida
- *  de la red neuronal multicapa.
+/** \fn PdsNnNatural pds_neuronml_training_results_printf(PdsNeuronML *MLayer,const char* Type)
+ *  \brief Muestra los resultados del entrenamiento de la red neuronal multicapa NeuronML.
  *
- *  Se considera como un voto positivo "+1" cualquier valor mayor o igual a "0". 
- *  Se considera como un voto negativo "-1" cualquier valor menor a "0". 
- *  Luego se realiza la suma con signo de las votaciones.
- *  \param[in] NeuronML Red neuronal Multi capa a trabajar.
- *  \param[out] Vote Valor de la votación abrupta.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
+ *  \param[in] MLayer Red neuronal Multi capa a trabajar.
+ *  \param[in] Type Etiqueda del entrenamiento.
+ *  \return El valor medio de las salidas de la red neuronal multicapa NeuronML.
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_get_hard_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
+PdsNnNatural pds_neuronml_training_results_printf(PdsNeuronML *MLayer,const char* Type)
 {
-	*Vote=0;
+    PdsNnNatural IterN;
+    PdsNnReal Err;
+    if(MLayer==NULL)    return 0;
 
-	if(NeuronML==NULL)		return FALSE;
+    pds_neuronml_get_number_iterations(MLayer,&IterN);
+    pds_neuronml_get_output_rms_error(MLayer,&Err);
 
-	pds_nvector_get_hard_vote(NeuronML->Layer[NeuronML->N-1],Vote);
+
+    printf("[%s] %6dIters RMSE=%6.3f\t",Type,IterN,Err);
+
+    PdsVector *Y=NULL;
+    Y=pds_vector_new(MLayer->L[MLayer->N]);
+    pds_neuronml_get_output(MLayer,Y);
+    printf("Y= {\t");
+    for (int i=0;i<Y->Nel;i++)
+    {
+        printf("%5.3f\t",Y->V[i]);
+    }
+    printf("}\n");
+    pds_vector_destroy(&Y);
+
+    return IterN;
+}
+
+/** \fn int pds_neuronml_get_output(const PdsNeuronML *NeuronML,PdsVector *Y)
+ *  \brief Devuelve la salida Y de la red neuronal.
+ *  \param[in] NeuronML Red neuronal Multi capa a trabajar.
+ *  \param[out] Y Vector donde se cargará los valores de la salida Y.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o Y==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_output(const PdsNeuronML *NeuronML,PdsVector *Y)
+{
+	if(NeuronML==NULL)	return FALSE;
+	if(Y==NULL)		    return FALSE;
+
+	pds_nvector_get_output(NeuronML->Layer[NeuronML->N-1],Y);
 
 	return TRUE;
 }
 
-
-/** \fn int pds_neuronml_get_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
+/** \fn int pds_neuronml_get_output_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
  *  \brief Evalúa una votación con desición suave de los valores de salida
  *  de la red neuronal multicapa. 
  *
@@ -664,7 +710,7 @@ int pds_neuronml_get_hard_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_get_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
+int pds_neuronml_get_output_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
 {
 	*Vote=0;
 
@@ -676,8 +722,232 @@ int pds_neuronml_get_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
 }
 
 
-/** \fn int pds_neuronml_fprintf(const PdsNeuronML *NeuronML, FILE *fd)
- *  \brief Guarda en un archivo de texto los pesos W[i], el valor de U y Sigma.
+
+/** \fn int pds_neuronml_get_output_rms_error(const PdsNeuronML *NeuronML,PdsNnReal *RmsError)
+ *  \brief Evalúa el error cuadrático medio de todos los errores Y[1] de las
+ *  neuronas de salida de la red neuronal.
+ *  \param[in] NeuronML Red neuronal Multi capa a trabajar.
+ *  \param[in] RmsError Valor raíz cuadrático medio.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_output_rms_error(const PdsNeuronML *NeuronML,PdsNnReal *RmsError)
+{
+	*RmsError=0;
+
+	if(NeuronML==NULL)		return FALSE;
+
+	pds_nvector_get_rms_error(NeuronML->Layer[NeuronML->N-1],RmsError);
+
+	return TRUE;
+}
+
+
+/** \fn int pds_neuronml_head_printf(const PdsNeuronML *NeuronML)
+ *  \brief Mostra em pantalla los datos de cabecera de la red neuronal multicapa
+ *  \param[in] NeuronML La estructura multicapa a leer.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_head_printf(const PdsNeuronML *NeuronML)
+{
+	if(NeuronML==NULL)		return FALSE;
+
+    printf("Number of layers:\t%d\n",NeuronML->N);
+
+    printf("Number of neurons by layer L:\n");
+    for(int i=0;i<=NeuronML->N;i++)
+    {
+        printf("L[%d]=%d",i,NeuronML->L[i]);
+        if(i==NeuronML->N)  printf("\n");
+        else                printf("\t");
+    }
+
+    printf("Maximum accepted RMS of output error:\t%f\n",NeuronML->MaxError);
+
+    printf("Trainning factor alpha:\t%f\n",NeuronML->Alpha);
+
+    printf("Maximum number of iterations:\t%d\n",NeuronML->MaxIter);
+
+	/*! Habilitador de printf de avance. 1 habilitado 0 deshabilitado*/
+	//PdsNnBool ShowPrintf;
+
+    return TRUE;
+}
+
+
+/** \fn int pds_neuronml_head_fprintf(const PdsNeuronML *NeuronML, FILE *fd)
+ *  \brief Escribe los datos de cabecera de la red neuronal.
+ *  En una linea separando los elementos por un TAB.
+ *  {NeuronML->N,NeuronML->MaxError,NeuronML->Alpha,NeuronML->MaxIter,NeuronML->Iter,NeuronML->ShowPrintf}
+ *  En la siguiente linea separando los elementos por un TAB, el numero de neuronas por capa.
+ *  \param[in] NeuronML La estructura multicapa a leer.
+ *  \param[in,out] fd Manejador del fichero a escribir.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o fd==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_head_fprintf(const PdsNeuronML *NeuronML, FILE *fd)
+{
+    if(NeuronML==NULL)  return FALSE;
+    if(fd==NULL)        return FALSE;
+
+	fprintf(fd,"%u\t",NeuronML->N);
+
+	fprintf(fd,"%e\t",NeuronML->MaxError);
+
+	fprintf(fd,"%e\t",NeuronML->Alpha);
+
+	fprintf(fd,"%u\t",NeuronML->MaxIter);
+
+	fprintf(fd,"%u\t",NeuronML->Iter);
+
+	fprintf(fd,"%u\n",NeuronML->ShowPrintf);
+
+	for(PdsNnNatural i=0;i<=NeuronML->N;i++)
+	{	
+		if(i!=NeuronML->N)	fprintf(fd,"%u\t",NeuronML->L[i]);
+		else			    fprintf(fd,"%u\n",NeuronML->L[i]);
+	}
+
+    return TRUE;
+}
+
+
+/** \fn PdsNeuronML *pds_neuronml_new_head_fscanf(FILE *fd)
+ *  \brief Lee los datos de cabecera de una red neuronal desde un archivo
+ *  \param[in,out] fd Manejador del fichero a escribir.
+ *  \return Un puntero al vector de tipo PdsNeuronML o NULL en caso de error.
+ *  \ingroup PdsNeuronMLGroup
+ */
+PdsNeuronML *pds_neuronml_new_head_fscanf(FILE *fd)
+{
+	PdsNeuronML *NeuronML=NULL;
+	PdsNnNatural i,j;
+	PdsNnReal x;
+	int tmp;
+
+	if(fd==NULL)	return NULL;
+
+	// Reservo memoria de la estructura.
+	NeuronML=(PdsNeuronML *)calloc(1,sizeof(PdsNeuronML));
+	if(NeuronML==NULL)	return NULL;
+
+	if(feof(fd)==0)	
+	{
+		tmp=fscanf(fd,"%e",&x);
+		if(tmp==0)	{free(NeuronML);	return NULL;}
+		NeuronML->N=(PdsNnNatural)x;
+	}
+	else		{free(NeuronML);	return NULL;}
+
+	if(feof(fd)==0)	
+	{
+		tmp=fscanf(fd,"%e",&(NeuronML->MaxError));
+		if(tmp==0)	{free(NeuronML);	return NULL;}
+	}
+	else		{free(NeuronML);	return NULL;}
+
+	if(feof(fd)==0)	
+	{
+		tmp=fscanf(fd,"%e",&(NeuronML->Alpha));
+		if(tmp==0)	{free(NeuronML);	return NULL;}
+	}
+	else		{free(NeuronML);	return NULL;}
+
+	if(feof(fd)==0)	
+	{
+		tmp=fscanf(fd,"%e",&x);
+		if(tmp==0)	{free(NeuronML);	return NULL;}
+		NeuronML->MaxIter=(PdsNnNatural)x;
+	}
+	else		{free(NeuronML);	return NULL;}
+
+	if(feof(fd)==0)	
+	{
+		tmp=fscanf(fd,"%e",&x);
+		if(tmp==0)	{free(NeuronML);	return NULL;}
+		NeuronML->Iter=(PdsNnNatural)x;
+	}
+	else		{free(NeuronML);	return NULL;}
+
+	if(feof(fd)==0)	
+	{
+		tmp=fscanf(fd,"%e",&x);
+		if(tmp==0)	{free(NeuronML);	return NULL;}
+		NeuronML->ShowPrintf=(PdsNnBool)x;
+	}
+	else		{free(NeuronML);	return NULL;}
+
+	// Reservo vector de capas.
+	NeuronML->L=(PdsNnNatural*)calloc(NeuronML->N+1,sizeof(PdsNnNatural));
+
+	if(NeuronML->L==NULL)	{free(NeuronML);	return NULL;}
+
+
+	for(i=0;i<=NeuronML->N;i++)
+	{
+		x=0;
+		if(feof(fd)==0)	
+		{
+			tmp=fscanf(fd,"%e",&x);
+			if(tmp==0)	
+			{free(NeuronML->L);	free(NeuronML);	return NULL;}
+		}
+		else	{free(NeuronML->L);	free(NeuronML);	return NULL;}
+
+
+		if(fabs(x)>1.0) NeuronML->L[i]=(PdsNnNatural)fabs(x);
+		else		    NeuronML->L[i]=1;
+
+	}
+
+	// Creo vector de neuronas de entrada.
+	NeuronML->LayerInput=pds_nivector_new(NeuronML->L[0]);
+	if(NeuronML->LayerInput==NULL)	
+	{
+		free(NeuronML->L);
+		free(NeuronML);
+		return NULL;
+	}
+
+
+	// Creo vector de neuronas de capas oculta y de salida.
+	NeuronML->Layer=(PdsNVector **)calloc(NeuronML->N,sizeof(PdsNVector *));
+	if(NeuronML->Layer==NULL) 
+	{
+		free(NeuronML->L);
+		pds_nivector_free(NeuronML->LayerInput);
+		free(NeuronML);
+		return NULL;
+	}
+
+	for(i=0;i<NeuronML->N;i++)
+	{	
+		NeuronML->Layer[i]=pds_nvector_new(NeuronML->L[i+1],NeuronML->L[i]);
+		if(NeuronML->Layer[i]==NULL)
+		{
+			for(j=0;j<i;j++)	pds_nvector_free(NeuronML->Layer[i]);
+			free(NeuronML->L);
+			pds_nivector_free(NeuronML->LayerInput);
+			free(NeuronML);
+			return NULL;
+		}
+		
+	}
+
+	pds_nvector_connect_input_with_nivector(NeuronML->Layer[0],NeuronML->LayerInput);
+	for(i=1;i<NeuronML->N;i++)
+	{
+		pds_nvector_connect_input(NeuronML->Layer[i],NeuronML->Layer[i-1]);
+	}
+
+    return NeuronML;
+}
+
+
+
+/** \fn int pds_neuronml_body_fprintf(const PdsNeuronML *NeuronML, FILE *fd)
+ *  \brief Guarda en un archivo de texto los pesos W[i], el valor de U.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  (neurona por neurona)
  *  \param[in] NeuronML La estructura multicapa a leer.
@@ -685,7 +955,7 @@ int pds_neuronml_get_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
  *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o fd==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_fprintf(const PdsNeuronML *NeuronML, FILE *fd)
+int pds_neuronml_body_fprintf(const PdsNeuronML *NeuronML, FILE *fd)
 {
 	PdsNnNatural i;
 	int m;
@@ -703,8 +973,8 @@ int pds_neuronml_fprintf(const PdsNeuronML *NeuronML, FILE *fd)
 }
 
 
-/** \fn int pds_neuronml_fscanf(PdsNeuronML *NeuronML, FILE *fd)
- *  \brief Lee de un archivo de texto los pesos W[i], el valor de U  y Sigma.
+/** \fn int pds_neuronml_body_fscanf(PdsNeuronML *NeuronML, FILE *fd)
+ *  \brief Lee de un archivo de texto los pesos W[i], el valor de U.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  (neurona por neurona)
  *  \param[out] NeuronML La estructura multicapa a escribir.
@@ -712,7 +982,7 @@ int pds_neuronml_fprintf(const PdsNeuronML *NeuronML, FILE *fd)
  *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o fd==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_fscanf(PdsNeuronML *NeuronML, FILE *fd)
+int pds_neuronml_body_fscanf(PdsNeuronML *NeuronML, FILE *fd)
 {
 	PdsNnNatural i;
 	int m;
@@ -731,15 +1001,15 @@ int pds_neuronml_fscanf(PdsNeuronML *NeuronML, FILE *fd)
 }
 
 
-/** \fn int pds_neuronml_fwrite(const PdsNeuronML *NeuronML, FILE *fd)
- *  \brief Guarda en un archivo binario los pesos W[i], el valor de U y Sigma.
+/** \fn int pds_neuronml_body_fwrite(const PdsNeuronML *NeuronML, FILE *fd)
+ *  \brief Guarda en un archivo binario los pesos W[i], el valor de U.
  *  (neurona por neurona)
  *  \param[in] NeuronML La estructura multicapa a leer.
  *  \param[in,out] fd Manejador del fichero binario a escribir.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o fd==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_fwrite(const PdsNeuronML *NeuronML, FILE *fd)
+int pds_neuronml_body_fwrite(const PdsNeuronML *NeuronML, FILE *fd)
 {
 	PdsNnNatural i;
 	int m;
@@ -757,15 +1027,15 @@ int pds_neuronml_fwrite(const PdsNeuronML *NeuronML, FILE *fd)
 }
 
 
-/** \fn int pds_neuronml_fread(PdsNeuronML *NeuronML, FILE *fd)
- *  \brief Lee de un archivo binario los pesos W[i], el valor de U  y Sigma.
+/** \fn int pds_neuronml_body_fread(PdsNeuronML *NeuronML, FILE *fd)
+ *  \brief Lee de un archivo binario los pesos W[i], el valor de U.
  *  (neurona por neurona)
  *  \param[out] NeuronML La estructura multicapa a escribir.
  *  \param[in,out] fd Manejador del fichero binario a escribir.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o fd==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_fread(PdsNeuronML *NeuronML, FILE *fd)
+int pds_neuronml_body_fread(PdsNeuronML *NeuronML, FILE *fd)
 {
 	PdsNnNatural i;
 	int m;
