@@ -30,7 +30,7 @@
 
 /** \fn PdsNVector *pds_nvector_new(PdsNnNatural Nel,PdsNnNatural Nd)
  *  \brief Crea un vector de neuronas de tipo PdsNVector e inicia con cero todos 
- *  los pesos,  U=1.0 e Sigma=1.0.
+ *  los pesos,  U=1.0.
  *  \param[in] Nel Es el número de elementos del vector.
  *  \param[in] Nd Es el número de entradas de cada neurona del vector.
  *  \return Un puntero al vector de tipo PdsNVector.
@@ -100,29 +100,27 @@ int pds_nvector_init_weight_uniform(PdsNVector *NVector,PdsUniform *RV)
 	return TRUE;
 }
 
-
-/** \fn int pds_nvector_set_sigma(PdsNVector *NVector,PdsNnReal Sigma)
- *  \brief Coloca el valor de  sigma de las Neuronas.
- *  \param[in,out] NVector El vector de neuronas a escribir.
- *  \param[in] Sigma El valor de sigma de las neuronas.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
+/** \fn int pds_nvector_init_weight_rand(PdsNVector *NVector,PdsSnReal min, PdsSnReal max)
+ *  \brief Inicia aleatoriamente siguiendo una distribución uniforme entre min e max
+ *  en todas las neuronas del vector, usando la funcion rand.
+ *  \param[in,out] NVector El vector a escribir.
+ *  \param[in] min valor mínimo.
+ *  \param[in] max valor máximo.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNVectorGroup
  */
-int pds_nvector_set_sigma(PdsNVector *NVector,PdsNnReal Sigma)
+int pds_nvector_init_weight_rand(PdsNVector *NVector,PdsSnReal min, PdsSnReal max)
 {
 	PdsNnNatural i;
 
 	if(NVector==NULL)	return FALSE;
 
 	for(i=0;i<NVector->Nel;i++)
-	NVector->V[i]->Sigma=Sigma;
+        pds_neuron_init_weight_rand(NVector->V[i],min,max);
 
 	return TRUE;
 }
 
-
-
-
 /** \fn int pds_nvector_init_u_uniform(PdsNVector *NVector,PdsUniform *RV)
  *  \brief Coloca de forma aleatoria siguiendo una distribucion uniforme, el 
  *  valor de  U de las Neuronas.
@@ -148,6 +146,29 @@ int pds_nvector_init_u_uniform(PdsNVector *NVector,PdsUniform *RV)
 	return TRUE;
 }
 
+/** \fn int pds_nvector_init_u_rand(PdsNVector *NVector,PdsSnReal min, PdsSnReal max)
+ *  \brief Coloca de forma aleatoria valores entre min y max, en el 
+ *  valor de  U de todas las Neuronas, es usada la funcion rand.
+ *  \param[in,out] NVector El vector de neurons a escribir.
+ *  \param[in] min valor mínimo.
+ *  \param[in] max valor máximo.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
+ *  \ingroup PdsNVectorGroup
+ */
+int pds_nvector_init_u_rand(PdsNVector *NVector,PdsSnReal min, PdsSnReal max)
+{
+	PdsNnNatural i;
+
+	if(NVector==NULL)	return FALSE;
+
+	for(i=0;i<NVector->Nel;i++)
+	{
+		NVector->V[i]->U=min+(rand()*(max-min))/RAND_MAX;
+	}
+
+	return TRUE;
+}
+
 
 /** \fn int pds_nvector_get_output(const PdsNVector *NVector,PdsVector *Y)
  *  \brief Devuelve el valor de la salida Y de las Neuronas.
@@ -262,7 +283,7 @@ int pds_nvector_evaluate_theta(PdsNVector *NVector)
 
 /** \fn int pds_nvector_evaluate_func(PdsNVector *NVector)
  *  \brief Evalua la funcion de activación de las Neuronas de NVector.
- *  \f[ func\left(\frac{\theta_j}{\sigma_j}\right) \f]
+ *  \f[ func\left(\theta_j\right) \f]
  *  \param[in,out] NVector El vector de neuronas a evaluar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
@@ -284,65 +305,17 @@ int pds_nvector_evaluate_func(PdsNVector *NVector)
 }
 
 
-/** \fn int pds_nvector_evaluate(PdsNVector *NVector)
- *  \brief Evalua las neuronas del vector NVector.
- *  \f[ \theta_j=\sum_{k=0}^{N_{d}-1}{W_{kj}X_k} - U_j \f]
- *  \f[ func\left(\frac{\theta_j}{\sigma_j}\right) \f]
- *  \param[in,out] NVector El vector de neuronas a evaluar.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
- *  \ingroup PdsNVectorGroup
- */
-int pds_nvector_evaluate(PdsNVector *NVector)
-{
-	int m;
-	if(NVector==NULL)		return FALSE;
-
-	m=pds_nvector_evaluate_theta(NVector);
-	if(m==FALSE)	return FALSE;
-	m=pds_nvector_evaluate_func(NVector);
-	if(m==FALSE)	return FALSE;
-
-	return TRUE;
-}
-
-
-/** \fn int pds_nvector_update(PdsNVector *NVector)
- *  \brief Actualiza el valor a la salida de las neuronas NVector
- *  \f[ y_j \leftarrow func\left(\frac{\theta_j}{\sigma_j}\right) \f]
- *  \param[in,out] NVector El vector de neuronas a trabajar.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
- *  \ingroup PdsNVectorGroup
- */
-int pds_nvector_update(PdsNVector *NVector)
-{
-	PdsNnNatural i;
-    int dat;
-
-	if(NVector==NULL)		return FALSE;
-
-	for(i=0;i<NVector->Nel;i++)
-	{
-        if((NVector->V[i])==NULL)  return FALSE;
-
-        dat=pds_neuron_update(NVector->V[i]);
-        if(dat==FALSE)  return FALSE;
-	}
-
-	return TRUE;
-}
-
 
 /** \fn int pds_nvector_iterate(PdsNVector *NVector)
  *  \brief Itera las neuronas del vector NVector.
  *  \f[ \theta_j=\sum_{k=0}^{N_{d}-1}{W_{kj}X_k} - U_j \f]
- *  \f[ y_j \leftarrow func\left(\frac{\theta_j}{\sigma_j}\right) \f]
+ *  \f[ y_j \leftarrow func\left(\theta_j\right) \f]
  *  \param[in,out] NVector El vector de neuronas a iterar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
  */
 int pds_nvector_iterate(PdsNVector *NVector)
 {
-	PdsNnNatural i;
 	int m;
 
 	if(NVector==NULL)		return FALSE;
@@ -352,14 +325,11 @@ int pds_nvector_iterate(PdsNVector *NVector)
 	m=pds_nvector_evaluate_func(NVector);
 	if(m==FALSE)	return FALSE;
 
-    m=pds_nvector_update(NVector);
-	if(m==FALSE)	return FALSE;
-
 	return TRUE;
 }
 
 
-/** \fn int pds_nvector_evaluate_error(PdsNVector *NVector,const PdsVector * R)
+/** \fn int pds_nvector_evaluate_diff_error(PdsNVector *NVector,const PdsVector * R)
  *  \brief Compara la salida de las neuronas del vector NVector con el vector de 
  *  referencia R, el resultado es cargado en la salida Y[1] de cada neurona del 
  *  vector NVector en  NVector->V[i]->Y[1].
@@ -370,10 +340,9 @@ int pds_nvector_iterate(PdsNVector *NVector)
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
  */
-int pds_nvector_evaluate_error(PdsNVector *NVector,const PdsVector * R)
+int pds_nvector_evaluate_diff_error(PdsNVector *NVector,const PdsVector * R)
 {
 	PdsNnNatural i;
-	int m;
 
 	if(NVector==NULL)		return FALSE;
 	if(NVector->Nel!= R->Nel)	return FALSE;
@@ -418,43 +387,11 @@ int pds_nvector_get_rms_error(const PdsNVector *NVector,PdsNnReal *RmsError)
 }
 
 
-/** \fn int pds_nvector_get_hard_vote(const PdsNVector *NVector,PdsNnReal *Vote)
- *  \brief Evalúa una votación de todos los valores de salida Y[0] de las
- *  neuronas.
- *
- *  Se considera como un voto positivo "+1" cualquier valor mayor o 
- *  igual a "0". Se considera como un voto negativo "-1" cualquier valor menor 
- *  a "0". Luego se realiza la suma con signo de las votaciones.
- *  \param[in,out] NVector El vector de neuronas a trabajar.
- *  \param[out] Vote Valor de la votación abrupta.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
- *  \ingroup PdsNVectorGroup
- */
-int pds_nvector_get_hard_vote(const PdsNVector *NVector,PdsNnReal *Vote)
-{
-	PdsNnNatural i;
-
-	*Vote=0.0;
-
-	if(NVector==NULL)		return FALSE;
-
-	for(i=0;i<NVector->Nel;i++)
-	{
-        if((NVector->V[i])==NULL)  return FALSE;
-
-		if(NVector->V[i]->Y[0]>=0)	(*Vote)=(*Vote)+1.0;
-		else				(*Vote)=(*Vote)-1.0;
-	}
-
-	return TRUE;
-}
-
-
 /** \fn int pds_nvector_get_soft_vote(const PdsNVector *NVector,PdsNnReal *Vote)
  *  \brief Evalúa una votación de todos los valores de salida Y[0] de las
  *  neuronas, se realiza una suma simple de todos los valores de salida.
  *  \param[in,out] NVector El vector de neuronas a trabajar.
- *  \param[out] Vote Valor de la votación suave.
+ *  \param[out] Vote Valor promedio de la votación suave.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
  */
@@ -473,22 +410,24 @@ int pds_nvector_get_soft_vote(const PdsNVector *NVector,PdsNnReal *Vote)
 		(*Vote)=(*Vote)+NVector->V[i]->Y[0];
 	}
 
+    (*Vote)=(*Vote)/NVector->Nel;
 	return TRUE;
 }
 
-/** \fn int pds_nvector_backpropagate_error(PdsNVector *NVector)
+/** \fn int pds_nvector_backpropagate_error(PdsNVector *NVector,PdsNnReal Alpha)
  *  \brief Retro propaga el error de las neuronas del vector NVector
  *  el resultado es cargado en la salida Y[1] de cada neurona del vector NVector
  *  conectada a las entradas.
- *  \f[ e_k \leftarrow e_k + \frac{func'\left(\frac{\theta_j}{\sigma_j}\right)}{\sigma_j} W_{kj} e_j \f]
+ *  \f[ e_k \leftarrow e_k + \frac{\alpha}{N_j} func'\left(\theta_j\right) W_{kj} e_j \f]
  *
  *  \image html backpropagation.png "Retropropagación del error en las neuronas." 
  *  \image html nvector_error_prop.png "Retropropagación del error en las neuronas." 
  *  \param[in,out] NVector El vector de neuronas a retropropagar el error.
+ *  \param[in] Alpha Factor de aprendizaje de los errores.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
  */
-int pds_nvector_backpropagate_error(PdsNVector *NVector)
+int pds_nvector_backpropagate_error(PdsNVector *NVector,PdsNnReal Alpha)
 {
 	PdsNnNatural i;
 	int m;
@@ -499,7 +438,7 @@ int pds_nvector_backpropagate_error(PdsNVector *NVector)
 	{
         if((NVector->V[i])==NULL)  return FALSE;
 
-		m=pds_neuron_backpropagate_error(NVector->V[i]);	
+		m=pds_neuron_backpropagate_error(NVector->V[i],Alpha);	
 		if(m==FALSE)	return FALSE;
 	}
 
@@ -507,9 +446,10 @@ int pds_nvector_backpropagate_error(PdsNVector *NVector)
 }
 
 
-/** \fn int pds_nvector_update_weight(PdsNVector *NVector, PdsNnReal Alpha)
+/** \fn int pds_nvector_update_u_and_weight(PdsNVector *NVector, PdsNnReal Alpha)
  *  \brief Actualiza los pesos W[k]j de las neuronas.
- *  \f[ \triangle W_{kj}= \alpha e_j \frac{func'\left(\frac{\theta_j}{\sigma_j}\right)}{\sigma_j} X_k \f]
+ *  \f[ \triangle W_{kj}= \alpha e_j func'\left(\theta_j\right) X_k \f]
+ *  \f[ \triangle U_{j}= -\alpha e_j func'\left(\theta_j\right) \f]
  *  "ej" es el error de la salida de la j-essima neurona.
  *  "func" es la función de activación da neurona.
  *  \param[in,out] NVector El vector de neuronas a actualizar los pesos.
@@ -517,7 +457,7 @@ int pds_nvector_backpropagate_error(PdsNVector *NVector)
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
  */
-int pds_nvector_update_weight(PdsNVector *NVector, PdsNnReal Alpha)
+int pds_nvector_update_u_and_weight(PdsNVector *NVector, PdsNnReal Alpha)
 {
 	PdsNnNatural i;
 	int m;
@@ -526,7 +466,7 @@ int pds_nvector_update_weight(PdsNVector *NVector, PdsNnReal Alpha)
 
 	for(i=0;i<NVector->Nel;i++)
 	{
-		m=pds_neuron_update_weight(NVector->V[i],Alpha);	
+		m=pds_neuron_update_u_and_weight(NVector->V[i],Alpha);	
 		if(m==FALSE)	return FALSE;
 	}
 
@@ -535,35 +475,9 @@ int pds_nvector_update_weight(PdsNVector *NVector, PdsNnReal Alpha)
 
 
 
-/** \fn int pds_nvector_update_weight_normalized(PdsNVector *NVector, PdsNnReal Alpha)
- *  \brief Actualiza los pesos W[k]j de las neuronas.
- *  \f[ \triangle W_{kj}= \alpha e_j \frac{func'\left(\frac{\theta_j}{\sigma_j}\right)}{\sigma_j} \frac{X_k}{X^TX} \f]
- *  "ej" es el error de la salida de la j-essima neurona.
- *  "func" es la función de activación da neurona.
- *  \param[in,out] NVector El vector de neuronas a actualizar los pesos.
- *  \param[in] Alpha Factor de aprendizaje de los pesos.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
- *  \ingroup PdsNVectorGroup
- */
-int pds_nvector_update_weight_normalized(PdsNVector *NVector, PdsNnReal Alpha)
-{
-	PdsNnNatural i;
-	int m;
-
-	if(NVector==NULL)		return FALSE;
-
-	for(i=0;i<NVector->Nel;i++)
-	{
-		m=pds_neuron_update_weight_normalized(NVector->V[i],Alpha);	
-		if(m==FALSE)	return FALSE;
-	}
-
-	return TRUE;
-}
-
 
 /** \fn int pds_nvector_fprintf(const PdsNVector *NVector, FILE *fd)
- *  \brief Guarda en un archivo de texto los pesos \f$W_{kj}\f$ y los valores \f$\{U_j,\sigma_j\}\f$.
+ *  \brief Guarda en un archivo de texto los pesos \f$W_{kj}\f$ y los valores \f$\{U_j\}\f$.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  \param[in] NVector El vector de neuronas a leer.
  *  \param[in,out] fd Manejador del fichero a escribir.
@@ -589,7 +503,7 @@ int pds_nvector_fprintf(const PdsNVector *NVector, FILE *fd)
 
 
 /** \fn int pds_nvector_fscanf(PdsNVector *NVector, FILE *fd)
- *  \brief Lee de un archivo de texto los pesos \f$W_{kj}\f$ y los valores \f$\{U_j,\sigma_j\}\f$.
+ *  \brief Lee de un archivo de texto los pesos \f$W_{kj}\f$ y los valores \f$\{U_j\}\f$.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  \param[out] NVector El vector a escribir.
  *  \param[in,out] fd Manejador del fichero a leer.
@@ -616,7 +530,7 @@ int pds_nvector_fscanf(PdsNVector *NVector, FILE *fd)
 
 
 /** \fn int pds_nvector_fwrite(const PdsNVector *NVector, FILE *fd)
- *  \brief Guarda en un archivo de texto binario los pesos \f$W_{kj}\f$ y los valores \f$\{U_j,\sigma_j\}\f$.
+ *  \brief Guarda en un archivo de texto binario los pesos \f$W_{kj}\f$ y los valores \f$\{U_j\}\f$.
  *  \param[in] NVector El vector de neuronas a leer.
  *  \param[in,out] fd Manejador del fichero binario a escribir.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL o fd==NULL). 
@@ -641,7 +555,7 @@ int pds_nvector_fwrite(const PdsNVector *NVector, FILE *fd)
 
 
 /** \fn int pds_nvector_fread(PdsNVector *NVector, FILE *fd)
- *  \brief Lee de un archivo de texto binario los pesos \f$W_{kj}\f$ y los valores \f$\{U_j,\sigma_j\}\f$.
+ *  \brief Lee de un archivo de texto binario los pesos \f$W_{kj}\f$ y los valores \f$\{U_j\}\f$.
  *  \param[out] NVector El vector a escribir.
  *  \param[in,out] fd Manejador del fichero binario a leer.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL o fd==NULL). 
