@@ -114,8 +114,7 @@ typedef struct
  *  <li>El máximo margen de error RMS aceptado en la salida de la estructura 
     PdsNeuronML es PDS_DEFAULT_MAX_RMS_ERROR.</li>
  *  <li>El valor de de Alpha se inicia con PDS_DEFAULT_ALPHA.</li>
- *  <li>El valor U se inicia con 1.0.</li>
- *  <li>El valor Sigma se inicia con 1.0.</li>
+ *  <li>El valor U y W son iniciados aleatoriamente entre -1.0 1.0.</li>
  *  <li>El valor func(x) se inicia con tanh(x/2).</li>
  *  </ul> 
  *  
@@ -125,6 +124,24 @@ typedef struct
  */
 PdsNeuronML *pds_neuronml_new(const PdsVector *L);
 
+/** \fn PdsNeuronML *pds_neuronml_new_from_array(const PdsNnNatural *L,PdsNnNatural Nel)
+ *  \brief Crea una estructura de tipo PdsNeuronML.
+ *
+ *  <ul>
+ *  <li>Inicia con cero todos los Pesos. </li>
+ *  <li>El máximo margen de error RMS aceptado en la salida de la estructura 
+    PdsNeuronML es PDS_DEFAULT_MAX_RMS_ERROR.</li>
+ *  <li>El valor de de Alpha se inicia con PDS_DEFAULT_ALPHA.</li>
+ *  <li>El valor U y W son iniciados aleatoriamente entre -1.0 1.0.</li>
+ *  <li>El valor func(x) se inicia con tanh(x/2).</li>
+ *  </ul> 
+ *  
+ *  \param[in] L Es un arreglo con el numero de elementos por capas.
+ *  \param[in] Nel Número de elementos del arreglo.
+ *  \return Un puntero al vector de tipo PdsNeuronML.
+ *  \ingroup PdsNeuronMLGroup
+ */
+PdsNeuronML *pds_neuronml_new_from_array(const PdsNnNatural *L,PdsNnNatural Nel);
 
 /** \fn PdsNeuronML *pds_neuronml_new_load_data(const char *mlayer_filename)
  *  \brief Crea una estructura de tipo PdsNeuronML e inicia todos los elementos.
@@ -140,7 +157,6 @@ PdsNeuronML *pds_neuronml_new(const PdsVector *L);
  *  <li>ShowPrintf, Si la estructura muestra información adicional.</li>
  *  <li>L, Vector de N+1 elementos con el número de neuronas por capa.</li>
  *  <li>Vector de pesos de neurona.</li>
- *  <li>Valor de U y Valor de Sigma</li>
  *  <li>...</li>
  *  </ul> 
  *  
@@ -164,7 +180,6 @@ PdsNeuronML *pds_neuronml_new_load_data(const char *mlayer_filename);
  *  <li>ShowPrintf, Si la estructura muestra información adicional.</li>
  *  <li>L, Vector de N+1 elementos con el número de neuronas por capa.</li>
  *  <li>Vector de pesos de neurona.</li>
- *  <li>Valor de U y Valor de Sigma</li>
  *  <li>...</li>
  *  </ul> 
  *  
@@ -178,7 +193,7 @@ int pds_neuronml_save_data(const PdsNeuronML *NeuronML, const char *mlayer_filen
 
 //@}
 
-/** @name pds_neuronml_<behavior_methods>
+/** @name pds_neuronml_<get_seT_configuration_methods>
  *  Métodos que alteran o describen el comportamiento de la estructura.
  *
  * @{
@@ -214,17 +229,6 @@ int pds_neuronml_set_max_error(PdsNeuronML *NeuronML,PdsNnReal MaxError);
 int pds_neuronml_set_max_iterations(PdsNeuronML *NeuronML,PdsNnNatural MaxIter);
 
 
-/** \fn int pds_neuronml_get_number_iterations(const PdsNeuronML *NeuronML,PdsNnNatural *Iter)
- *  \brief Devuelve el número de iteraciones del último entrenamiento hecho con
- *  la función pds_neuronml_training().
- *  \param[in,out] NeuronML Red neuronal Multi capa a trabajar.
- *  \param[in] Iter Número de iteraciones del último entrenamiento.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
- *  \ingroup PdsNeuronMLGroup
- */
-int pds_neuronml_get_number_iterations(const PdsNeuronML *NeuronML,PdsNnNatural *Iter);
-
-
 /** \fn int pds_neuronml_enable_printf(PdsNeuronML *NeuronML)
  *  \brief Habilita la muestra en pantalla del avance del aprendizaje.
  *  \param[in,out] NeuronML Red neuronal Multi capa a trabajar.
@@ -233,10 +237,45 @@ int pds_neuronml_get_number_iterations(const PdsNeuronML *NeuronML,PdsNnNatural
  */
 int pds_neuronml_enable_printf(PdsNeuronML *NeuronML);
 
+/** \fn int pds_neuronml_get_number_of_input_neurons(PdsNeuronML *NeuronML,PdsNnNatural *Number)
+ *  \brief Retorna o numero de neuronas de entrada.
+ *  \param[in] NeuronML Red neuronal Multi capa a cosultar.
+ *  \param[out] Number Número de neuronas de entrada.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_number_of_input_neurons(PdsNeuronML *NeuronML,PdsNnNatural *Number);
+
+/** \fn int pds_neuronml_get_number_of_output_neurons(PdsNeuronML *NeuronML,PdsNnNatural *Number)
+ *  \brief Retorna o numero de neuronas de salida.
+ *  \param[in] NeuronML Red neuronal Multi capa a cosultar.
+ *  \param[out] Number Número de neuronas de salida.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_number_of_output_neurons(PdsNeuronML *NeuronML,PdsNnNatural *Number);
+
+/** \fn int pds_neuronml_get_maximum_output(PdsNeuronML *NeuronML,PdsNnReal *Max)
+ *  \brief Retorna o valor maximo de las neuronas de salida
+ *  \param[in] NeuronML Red neuronal Multi capa a cosultar.
+ *  \param[out] Max Valor máximo de las neuronas de salida.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_maximum_output(PdsNeuronML *NeuronML,PdsNnReal *Max);
+
+/** \fn int pds_neuronml_get_minimum_output(PdsNeuronML *NeuronML,PdsNnReal *Min)
+ *  \brief Retorna o valor maximo de las neuronas de salida
+ *  \param[in] NeuronML Red neuronal Multi capa a cosultar.
+ *  \param[out] Min Valor mínimo de las neuronas de salida.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_minimum_output(PdsNeuronML *NeuronML,PdsNnReal *Min);
 
 //@}
 
-/** @name pds_neuronml_<variable_methods>
+/** @name pds_neuronml_<u_weight_methods>
  *  Métodos que alteran las variables de la estructura.
  *
  * @{
@@ -252,6 +291,17 @@ int pds_neuronml_enable_printf(PdsNeuronML *NeuronML);
  */
 int pds_neuronml_init_weight_uniform(PdsNeuronML *NeuronML,PdsUniform *RV);
 
+/** \fn int pds_neuronml_init_weight_rand(PdsNeuronML *NeuronML,PdsSnReal min, PdsSnReal max)
+ *  \brief Inicializa todos los pesos entre los valores min y max, 
+ *  usando internamente la funcion rand.
+ *  \param[in,out] NeuronML Red neuronal Multi capa a trabajar.
+ *  \param[in] min Valor mínimo.
+ *  \param[in] max Valor máximo.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_init_weight_rand(PdsNeuronML *NeuronML,PdsSnReal min, PdsSnReal max);
+
 
 /** \fn int pds_neuronml_init_u_uniform(PdsNeuronML *NeuronML,PdsUniform *RV)
  *  \brief Inicializa todos los umbrales U usando una variable aleatoria uniforme.
@@ -263,15 +313,15 @@ int pds_neuronml_init_weight_uniform(PdsNeuronML *NeuronML,PdsUniform *RV);
 int pds_neuronml_init_u_uniform(PdsNeuronML *NeuronML,PdsUniform *RV);
 
 
-/** \fn int pds_neuronml_set_sigma(PdsNeuronML *NeuronML,PdsNnReal Sigma)
- *  \brief Coloca el valor de Sigma en todas las neuronas
+/** \fn int pds_neuronml_init_u_rand(PdsNeuronML *NeuronML,PdsSnReal min, PdsSnReal max)
+ *  \brief Inicializa todos los umbrales U de forma aleatoria entre min y max.
  *  \param[in,out] NeuronML Red neuronal Multi capa a trabajar.
- *  \param[in] Sigma Valor de sigma en la función de activación de las neuronas.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL, RV==NULL). 
+ *  \param[in] min Valor mínimo.
+ *  \param[in] max Valor máximo.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_set_sigma(PdsNeuronML *NeuronML,PdsNnReal Sigma);
-
+int pds_neuronml_init_u_rand(PdsNeuronML *NeuronML,PdsSnReal min, PdsSnReal max);
 
 //@}
 
@@ -285,7 +335,7 @@ int pds_neuronml_set_sigma(PdsNeuronML *NeuronML,PdsNnReal Sigma);
 /** \fn int pds_neuronml_iterate(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector *Output)
  *  \brief Itera la red neuronal multicapa NeuronML.
  *  \f[ \theta_j=\sum_{k=0}^{N_{d}-1}{W_{kj}X_k} - U_j \f]
- *  \f[ y_j \leftarrow func\left(\frac{\theta_j}{\sigma_j}\right) \f]
+ *  \f[ y_j \leftarrow func\left(\theta_j\right) \f]
  *  \param[in,out] NeuronML Red neuronal Multi capa a trabajar.
  *  \param[in] Input Datos de entrada de la red neuronal multicapa.
  *  \param[out] Output Datos de salida de la red neuronal multicapa.
@@ -295,23 +345,38 @@ int pds_neuronml_set_sigma(PdsNeuronML *NeuronML,PdsNnReal Sigma);
  */
 int pds_neuronml_iterate(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector *Output);
 
-
-/** \fn int pds_neuronml_get_hard_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
- *  \brief Evalúa una votación con desición abrupta de los valores de salida
- *  de la red neuronal multicapa.
- *
- *  Se considera como un voto positivo "+1" cualquier valor mayor o igual a "0". 
- *  Se considera como un voto negativo "-1" cualquier valor menor a "0". 
- *  Luego se realiza la suma con signo de las votaciones.
- *  \param[in] NeuronML Red neuronal Multi capa a trabajar.
- *  \param[out] Vote Valor de la votación abrupta.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
+/** \fn int pds_neuronml_get_number_iterations(const PdsNeuronML *NeuronML,PdsNnNatural *Iter)
+ *  \brief Devuelve el número de iteraciones del último entrenamiento hecho con
+ *  la función pds_neuronml_training().
+ *  \param[in,out] NeuronML Red neuronal Multi capa a trabajar.
+ *  \param[in] Iter Número de iteraciones del último entrenamiento.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_get_hard_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote);
+int pds_neuronml_get_number_iterations(const PdsNeuronML *NeuronML,PdsNnNatural *Iter);
 
 
-/** \fn int pds_neuronml_get_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
+/** \fn PdsNnBool pds_neuronml_iterate_results_printf(const PdsNeuronML *MLayer,const char* Type)
+ *  \brief Muestra los resultados de la iteración de la red neuronal multicapa NeuronML.
+ *
+ *  \param[in] MLayer Red neuronal Multi capa a trabajar.
+ *  \param[in] Type Etiqueda da iteracion.
+ *  \return El valor medio de las salidas de la red neuronal multicapa NeuronML.
+ *  \ingroup PdsNeuronMLGroup
+ */
+PdsNnBool pds_neuronml_iterate_results_printf(const PdsNeuronML *MLayer,const char* Type);
+
+/** \fn int pds_neuronml_get_output(const PdsNeuronML *NeuronML,PdsVector *Y)
+ *  \brief Devuelve la salida Y de la red neuronal.
+ *  \param[in] NeuronML Red neuronal Multi capa a trabajar.
+ *  \param[out] Y Vector donde se cargará los valores de la salida Y.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o Y==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_output(const PdsNeuronML *NeuronML,PdsVector *Y);
+
+
+/** \fn int pds_neuronml_get_output_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote)
  *  \brief Evalúa una votación con desición suave de los valores de salida
  *  de la red neuronal multicapa. 
  *
@@ -321,7 +386,19 @@ int pds_neuronml_get_hard_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote);
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_get_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote);
+int pds_neuronml_get_output_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote);
+
+
+
+/** \fn int pds_neuronml_get_output_rms_error(const PdsNeuronML *NeuronML,PdsNnReal *RmsError)
+ *  \brief Evalúa el error cuadrático medio de todos los errores Y[1] de las
+ *  neuronas de salida de la red neuronal.
+ *  \param[in] NeuronML Red neuronal Multi capa a trabajar.
+ *  \param[in] RmsError Valor raíz cuadrático medio.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_get_output_rms_error(const PdsNeuronML *NeuronML,PdsNnReal *RmsError);
 
 //@}
 
@@ -337,9 +414,10 @@ int pds_neuronml_get_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote);
  *
  *  Bucle de retropropagación e iteración :
  *  <ul>
- *  <li>pds_nvector_evaluate_error()</li>
- *  <li>pds_nvector_backpropagate_error() : \f$ e_k \leftarrow e_k + \frac{func'\left(\frac{\theta_j}{\sigma_j}\right)}{\sigma_j} W_{kj} e_j \f$ </li>
- *  <li>pds_nvector_update_weight() : \f$ W_{kj} = W_{kj} + \alpha e_j \frac{func'\left(\frac{\theta_j}{\sigma_j}\right)}{\sigma_j} \frac{X_{k}}{X^TX} \f$</li>
+ *  <li>pds_nvector_evaluate_diff_error()</li>
+ *  <li>pds_nvector_backpropagate_error() : \f$ e_k \leftarrow e_k + \frac{\alpha}{N_j} func'\left(\theta_j\right) W_{kj} e_j \f$ </li>
+ *  <li>pds_nvector_update_u_and_weight() : \f$ W_{kj} = W_{kj} + \alpha e_j func'\left(\theta_j\right) \frac{X_{k}}{X^TX} \f$</li>
+ *  <li>pds_nvector_update_u_and_weight() : \f$ U_{k} = U_{k} - \alpha e_j func'\left(\theta_j\right) \frac{1}{X^TX} \f$</li>
  *  <li>pds_neuronml_iterate()</li>
  *  </ul>
  *  El bucle es repetido hasta que el error cuadratico medio en la salida sea menor
@@ -354,6 +432,15 @@ int pds_neuronml_get_soft_vote(const PdsNeuronML *NeuronML,PdsNnReal *Vote);
  */
 int pds_neuronml_training(PdsNeuronML *NeuronML,const PdsVector *Input,PdsVector *Output);
 
+/** \fn PdsNnNatural pds_neuronml_training_results_printf(PdsNeuronML *MLayer,const char* Type)
+ *  \brief Muestra los resultados del entrenamiento de la red neuronal multicapa NeuronML.
+ *
+ *  \param[in] MLayer Red neuronal Multi capa a trabajar.
+ *  \param[in] Type Etiqueda del entrenamiento.
+ *  \return El valor medio de las salidas de la red neuronal multicapa NeuronML.
+ *  \ingroup PdsNeuronMLGroup
+ */
+PdsNnNatural pds_neuronml_training_results_printf(PdsNeuronML *MLayer,const char* Type);
 
 /** \fn int pds_neuronml_set_alpha(PdsNeuronML *NeuronML,PdsNnReal Alpha)
  *  \brief Coloca el valor del factor de aprendizaje Alpha.
@@ -367,15 +454,45 @@ int pds_neuronml_set_alpha(PdsNeuronML *NeuronML,PdsNnReal Alpha);
 
 //@}
 
+
 /** @name pds_neuronml_<fprintf>
- *  Funciones para leer y escribir datos de {Pesos, U, Sigma} de todas las neuronas.
+ *  Funciones para leer y escribir datos de {Pesos, U} de todas las neuronas.
  *
  * @{
  */
 
 
-/** \fn int pds_neuronml_fprintf(const PdsNeuronML *NeuronML, FILE *fd)
- *  \brief Guarda en un archivo de texto los pesos W[i], el valor de U y Sigma.
+/** \fn int pds_neuronml_head_printf(const PdsNeuronML *NeuronML)
+ *  \brief Mostra em pantalla los datos de cabecera de la red neuronal multicapa
+ *  \param[in] NeuronML La estructura multicapa a leer.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_head_printf(const PdsNeuronML *NeuronML);
+
+
+/** \fn int pds_neuronml_head_fprintf(const PdsNeuronML *NeuronML, FILE *fd)
+ *  \brief Escribe los datos de cabecera de la red neuronal.
+ *  En una linea separando los elementos por un TAB.
+ *  {NeuronML->N,NeuronML->MaxError,NeuronML->Alpha,NeuronML->MaxIter,NeuronML->Iter,NeuronML->ShowPrintf}
+ *  En la siguiente linea separando los elementos por un TAB, el numero de neuronas por capa.
+ *  \param[in] NeuronML La estructura multicapa a leer.
+ *  \param[in,out] fd Manejador del fichero a escribir.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o fd==NULL). 
+ *  \ingroup PdsNeuronMLGroup
+ */
+int pds_neuronml_head_fprintf(const PdsNeuronML *NeuronML, FILE *fd);
+
+/** \fn PdsNeuronML *pds_neuronml_new_head_fscanf(FILE *fd)
+ *  \brief Lee los datos de cabecera de una red neuronal desde un archivo
+ *  \param[in,out] fd Manejador del fichero a escribir.
+ *  \return Un puntero al vector de tipo PdsNeuronML o NULL en caso de error.
+ *  \ingroup PdsNeuronMLGroup
+ */
+PdsNeuronML *pds_neuronml_new_head_fscanf(FILE *fd);
+
+/** \fn int pds_neuronml_body_fprintf(const PdsNeuronML *NeuronML, FILE *fd)
+ *  \brief Guarda en un archivo de texto los pesos W[i], el valor de U.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  (neurona por neurona)
  *  \param[in] NeuronML La estructura multicapa a leer.
@@ -383,11 +500,11 @@ int pds_neuronml_set_alpha(PdsNeuronML *NeuronML,PdsNnReal Alpha);
  *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o fd==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_fprintf(const PdsNeuronML *NeuronML, FILE *fd);
+int pds_neuronml_body_fprintf(const PdsNeuronML *NeuronML, FILE *fd);
 
 
-/** \fn int pds_neuronml_fscanf(PdsNeuronML *NeuronML, FILE *fd)
- *  \brief Lee de un archivo de texto los pesos W[i], el valor de U  y Sigma.
+/** \fn int pds_neuronml_body_fscanf(PdsNeuronML *NeuronML, FILE *fd)
+ *  \brief Lee de un archivo de texto los pesos W[i], el valor de U.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  (neurona por neurona)
  *  \param[out] NeuronML La estructura multicapa a escribir.
@@ -395,29 +512,29 @@ int pds_neuronml_fprintf(const PdsNeuronML *NeuronML, FILE *fd);
  *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o fd==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_fscanf(PdsNeuronML *NeuronML, FILE *fd);
+int pds_neuronml_body_fscanf(PdsNeuronML *NeuronML, FILE *fd);
 
 
-/** \fn int pds_neuronml_fwrite(const PdsNeuronML *NeuronML, FILE *fd)
- *  \brief Guarda en un archivo binario los pesos W[i], el valor de U y Sigma.
+/** \fn int pds_neuronml_body_fwrite(const PdsNeuronML *NeuronML, FILE *fd)
+ *  \brief Guarda en un archivo binario los pesos W[i], el valor de U.
  *  (neurona por neurona)
  *  \param[in] NeuronML La estructura multicapa a leer.
  *  \param[in,out] fd Manejador del fichero binario a escribir.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o fd==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_fwrite(const PdsNeuronML *NeuronML, FILE *fd);
+int pds_neuronml_body_fwrite(const PdsNeuronML *NeuronML, FILE *fd);
 
 
-/** \fn int pds_neuronml_fread(PdsNeuronML *NeuronML, FILE *fd)
- *  \brief Lee de un archivo binario los pesos W[i], el valor de U  y Sigma.
+/** \fn int pds_neuronml_body_fread(PdsNeuronML *NeuronML, FILE *fd)
+ *  \brief Lee de un archivo binario los pesos W[i], el valor de U.
  *  (neurona por neurona)
  *  \param[out] NeuronML La estructura multicapa a escribir.
  *  \param[in,out] fd Manejador del fichero binario a escribir.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NeuronML==NULL o fd==NULL). 
  *  \ingroup PdsNeuronMLGroup
  */
-int pds_neuronml_fread(PdsNeuronML *NeuronML, FILE *fd);
+int pds_neuronml_body_fread(PdsNeuronML *NeuronML, FILE *fd);
 
 //@}
 
