@@ -24,13 +24,13 @@
 #ifndef __PDSNN_H__
 #define __PDSNN_H__
 
-#define PDS_DEFAULT_MAX_RMS_ERROR 0.70711
+#define PDS_DEFAULT_MAX_RMS_ERROR 0.4
 #define PDS_DEFAULT_ALPHA 0.5
-#define PDS_DEFAULT_MAX_ITER 0
+#define PDS_DEFAULT_MAX_ITER 1000000
 
 #include <pds/pdsnnglobal.h>
 
-#include <pds/pdsnivector.h>
+#include <pds/pdsgeneratedata.h>
 #include <pds/pdsnvector.h>
 #include <pds/pdsneuronml.h>
 
