@@ -90,7 +90,7 @@ typedef struct
 
 /** \fn PdsNVector *pds_nvector_new(PdsNnNatural Nel,PdsNnNatural Nd)
  *  \brief Crea un vector de neuronas de tipo PdsNVector e inicia con cero todos 
- *  los pesos,  U=1.0 e Sigma=1.0.
+ *  los pesos,  U=1.0.
  *  \param[in] Nel Es el número de elementos del vector.
  *  \param[in] Nd Es el número de entradas de cada neurona del vector.
  *  \return Un puntero al vector de tipo PdsNVector.
@@ -116,30 +116,42 @@ PdsNVector *pds_nvector_new(PdsNnNatural Nel,PdsNnNatural Nd);
  */
 int pds_nvector_init_weight_uniform(PdsNVector *NVector,PdsUniform *RV);
 
-
-//@}
-
-
-/** @name pds_nvector_<output_methods>
- *  Trabajando con funciones de saida
- *
- * @{
- */
-
-
-/** \fn int pds_nvector_get_output(const PdsNVector *NVector,PdsVector *Y)
- *  \brief Devuelve el valor de la salida Y de las Neuronas.
- *  \param[in] NVector El vector de neuronas en consulta.
- *  \param[out] Y Vector donde se cargará los valores de la salida Y.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL, Y==NULL o
- *  tamaños distintos). 
+/** \fn int pds_nvector_init_weight_rand(PdsNVector *NVector,PdsSnReal min, PdsSnReal max)
+ *  \brief Inicia aleatoriamente siguiendo una distribución uniforme entre min e max
+ *  en todas las neuronas del vector, usando la funcion rand.
+ *  \param[in,out] NVector El vector a escribir.
+ *  \param[in] min valor mínimo.
+ *  \param[in] max valor máximo.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: Neuron==NULL). 
  *  \ingroup PdsNVectorGroup
  */
-int pds_nvector_get_output(const PdsNVector *NVector,PdsVector *Y);
+int pds_nvector_init_weight_rand(PdsNVector *NVector,PdsSnReal min, PdsSnReal max);
 
+/** \fn int pds_nvector_init_u_uniform(PdsNVector *NVector,PdsUniform *RV)
+ *  \brief Coloca de forma aleatoria siguiendo una distribucion uniforme, el 
+ *  valor de  U de las Neuronas.
+ *  \param[in,out] NVector El vector de neuronas a escribir.
+ *  \param[in] RV Variable aleatoria uniforme.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
+ *  \ingroup PdsNVectorGroup
+ */
+int pds_nvector_init_u_uniform(PdsNVector *NVector,PdsUniform *RV);
+
+/** \fn int pds_nvector_init_u_rand(PdsNVector *NVector,PdsSnReal min, PdsSnReal max)
+ *  \brief Coloca de forma aleatoria valores entre min y max, en el 
+ *  valor de  U de todas las Neuronas, es usada la funcion rand.
+ *  \param[in,out] NVector El vector de neurons a escribir.
+ *  \param[in] min valor mínimo.
+ *  \param[in] max valor máximo.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
+ *  \ingroup PdsNVectorGroup
+ */
+int pds_nvector_init_u_rand(PdsNVector *NVector,PdsSnReal min, PdsSnReal max);
 
 //@}
 
+
+
 /** @name pds_nvector_<connect_methods>
  *  Trabajando con el metodos para conectar vectores de neuronas.
  *
@@ -199,17 +211,6 @@ int pds_nvector_connect_input_with_nivector(PdsNVector *NVector,PdsNIVector *X);
 int pds_nvector_evaluate_theta(PdsNVector *NVector);
 
 
-/** \fn int pds_nvector_init_u_uniform(PdsNVector *NVector,PdsUniform *RV)
- *  \brief Coloca de forma aleatoria siguiendo una distribucion uniforme, el 
- *  valor de  U de las Neuronas.
- *  \param[in,out] NVector El vector de neuronas a escribir.
- *  \param[in] RV Variable aleatoria uniforme.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
- *  \ingroup PdsNVectorGroup
- */
-int pds_nvector_init_u_uniform(PdsNVector *NVector,PdsUniform *RV);
-
-
 //@}
 
 /** @name pds_nvector_<func_methods>
@@ -220,7 +221,7 @@ int pds_nvector_init_u_uniform(PdsNVector *NVector,PdsUniform *RV);
 
 /** \fn int pds_nvector_evaluate_func(PdsNVector *NVector)
  *  \brief Evalua la funcion de activación de las Neuronas de NVector.
- *  \f[ func\left(\frac{\theta_j}{\sigma_j}\right) \f]
+ *  \f[ func\left(\theta_j\right) \f]
  *  \param[in,out] NVector El vector de neuronas a evaluar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
@@ -228,16 +229,6 @@ int pds_nvector_init_u_uniform(PdsNVector *NVector,PdsUniform *RV);
 int pds_nvector_evaluate_func(PdsNVector *NVector);
 
 
-/** \fn int pds_nvector_set_sigma(PdsNVector *NVector,PdsNnReal Sigma)
- *  \brief Coloca el valor de  sigma de las Neuronas.
- *  \param[in,out] NVector El vector de neuronas a escribir.
- *  \param[in] Sigma El valor de sigma de las neuronas.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
- *  \ingroup PdsNVectorGroup
- */
-int pds_nvector_set_sigma(PdsNVector *NVector,PdsNnReal Sigma);
-
-
 //@}
 
 /** @name pds_nvector_<evaluate_methods>
@@ -246,31 +237,10 @@ int pds_nvector_set_sigma(PdsNVector *NVector,PdsNnReal Sigma);
  * @{
  */
 
-/** \fn int pds_nvector_evaluate(PdsNVector *NVector)
- *  \brief Evalua las neuronas del vector NVector.
- *  \f[ \theta_j=\sum_{k=0}^{N_{d}-1}{W_{kj}X_k} - U_j \f]
- *  \f[ func\left(\frac{\theta_j}{\sigma_j}\right) \f]
- *  \param[in,out] NVector El vector de neuronas a evaluar.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
- *  \ingroup PdsNVectorGroup
- */
-int pds_nvector_evaluate(PdsNVector *NVector);
-
-
-/** \fn int pds_nvector_update(PdsNVector *NVector)
- *  \brief Actualiza el valor a la salida de las neuronas NVector
- *  \f[ y_j \leftarrow func\left(\frac{\theta_j}{\sigma_j}\right) \f]
- *  \param[in,out] NVector El vector de neuronas a trabajar.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
- *  \ingroup PdsNVectorGroup
- */
-int pds_nvector_update(PdsNVector *NVector);
-
-
 /** \fn int pds_nvector_iterate(PdsNVector *NVector)
  *  \brief Itera las neuronas del vector NVector.
  *  \f[ \theta_j=\sum_{k=0}^{N_{d}-1}{W_{kj}X_k} - U_j \f]
- *  \f[ y_j \leftarrow func\left(\frac{\theta_j}{\sigma_j}\right) \f]
+ *  \f[ y_j \leftarrow func\left(\theta_j\right) \f]
  *  \param[in,out] NVector El vector de neuronas a iterar.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
@@ -279,6 +249,26 @@ int pds_nvector_iterate(PdsNVector *NVector);
 
 
 
+//@}
+
+/** @name pds_nvector_<output_methods>
+ *  Trabajando con funciones de saida
+ *
+ * @{
+ */
+
+
+/** \fn int pds_nvector_get_output(const PdsNVector *NVector,PdsVector *Y)
+ *  \brief Devuelve el valor de la salida Y de las Neuronas.
+ *  \param[in] NVector El vector de neuronas en consulta.
+ *  \param[out] Y Vector donde se cargará los valores de la salida Y.
+ *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL, Y==NULL o
+ *  tamaños distintos). 
+ *  \ingroup PdsNVectorGroup
+ */
+int pds_nvector_get_output(const PdsNVector *NVector,PdsVector *Y);
+
+
 //@}
 
 /** @name pds_nvector_<others_methods>
@@ -288,7 +278,7 @@ int pds_nvector_iterate(PdsNVector *NVector);
  */
 
 
-/** \fn int pds_nvector_evaluate_error(PdsNVector *NVector,const PdsVector * R)
+/** \fn int pds_nvector_evaluate_diff_error(PdsNVector *NVector,const PdsVector * R)
  *  \brief Compara la salida de las neuronas del vector NVector con el vector de 
  *  referencia R, el resultado es cargado en la salida Y[1] de cada neurona del 
  *  vector NVector en  NVector->V[i]->Y[1].
@@ -299,7 +289,7 @@ int pds_nvector_iterate(PdsNVector *NVector);
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
  */
-int pds_nvector_evaluate_error(PdsNVector *NVector,const PdsVector * R);
+int pds_nvector_evaluate_diff_error(PdsNVector *NVector,const PdsVector * R);
 
 
 /** \fn int pds_nvector_get_rms_error(const PdsNVector *NVector,PdsNnReal *RmsError)
@@ -313,46 +303,31 @@ int pds_nvector_evaluate_error(PdsNVector *NVector,const PdsVector * R);
 int pds_nvector_get_rms_error(const PdsNVector *NVector,PdsNnReal *RmsError);
 
 
-/** \fn int pds_nvector_get_hard_vote(const PdsNVector *NVector,PdsNnReal *Vote)
- *  \brief Evalúa una votación de todos los valores de salida Y[0] de las
- *  neuronas.
- *
- *  Se considera como un voto positivo "+1" cualquier valor mayor o 
- *  igual a "0". Se considera como un voto negativo "-1" cualquier valor menor 
- *  a "0". Luego se realiza la suma con signo de las votaciones.
- *  \param[in,out] NVector El vector de neuronas a trabajar.
- *  \param[out] Vote Valor de la votación abrupta.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
- *  \ingroup PdsNVectorGroup
- */
-int pds_nvector_get_hard_vote(const PdsNVector *NVector,PdsNnReal *Vote);
-
-
 /** \fn int pds_nvector_get_soft_vote(const PdsNVector *NVector,PdsNnReal *Vote)
  *  \brief Evalúa una votación de todos los valores de salida Y[0] de las
  *  neuronas, se realiza una suma simple de todos los valores de salida.
  *  \param[in,out] NVector El vector de neuronas a trabajar.
- *  \param[out] Vote Valor de la votación suave.
+ *  \param[out] Vote Valor promedio de la votación suave.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
  */
 int pds_nvector_get_soft_vote(const PdsNVector *NVector,PdsNnReal *Vote);
 
 
-
-/** \fn int pds_nvector_backpropagate_error(PdsNVector *NVector)
+/** \fn int pds_nvector_backpropagate_error(PdsNVector *NVector,PdsNnReal Alpha)
  *  \brief Retro propaga el error de las neuronas del vector NVector
  *  el resultado es cargado en la salida Y[1] de cada neurona del vector NVector
  *  conectada a las entradas.
- *  \f[ e_k \leftarrow e_k + \frac{func'\left(\frac{\theta_j}{\sigma_j}\right)}{\sigma_j} W_{kj} e_j \f]
+ *  \f[ e_k \leftarrow e_k + \frac{\alpha}{N_j} func'\left(\theta_j\right) W_{kj} e_j \f]
  *
  *  \image html backpropagation.png "Retropropagación del error en las neuronas." 
  *  \image html nvector_error_prop.png "Retropropagación del error en las neuronas." 
  *  \param[in,out] NVector El vector de neuronas a retropropagar el error.
+ *  \param[in] Alpha Factor de aprendizaje de los errores.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
  */
-int pds_nvector_backpropagate_error(PdsNVector *NVector);
+int pds_nvector_backpropagate_error(PdsNVector *NVector,PdsNnReal Alpha);
 
 
 
@@ -365,9 +340,10 @@ int pds_nvector_backpropagate_error(PdsNVector *NVector);
  */
 
 
-/** \fn int pds_nvector_update_weight(PdsNVector *NVector, PdsNnReal Alpha)
+/** \fn int pds_nvector_update_u_and_weight(PdsNVector *NVector, PdsNnReal Alpha)
  *  \brief Actualiza los pesos W[k]j de las neuronas.
- *  \f[ \triangle W_{kj}= \alpha e_j \frac{func'\left(\frac{\theta_j}{\sigma_j}\right)}{\sigma_j} X_k \f]
+ *  \f[ \triangle W_{kj}= \alpha e_j func'\left(\theta_j\right) X_k \f]
+ *  \f[ \triangle U_{j}= -\alpha e_j func'\left(\theta_j\right) \f]
  *  "ej" es el error de la salida de la j-essima neurona.
  *  "func" es la función de activación da neurona.
  *  \param[in,out] NVector El vector de neuronas a actualizar los pesos.
@@ -375,23 +351,10 @@ int pds_nvector_backpropagate_error(PdsNVector *NVector);
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
  *  \ingroup PdsNVectorGroup
  */
-int pds_nvector_update_weight(PdsNVector *NVector, PdsNnReal Alpha);
+int pds_nvector_update_u_and_weight(PdsNVector *NVector, PdsNnReal Alpha);
 
 
 
-/** \fn int pds_nvector_update_weight_normalized(PdsNVector *NVector, PdsNnReal Alpha)
- *  \brief Actualiza los pesos W[k]j de las neuronas.
- *  \f[ \triangle W_{kj}= \alpha e_j \frac{func'\left(\frac{\theta_j}{\sigma_j}\right)}{\sigma_j} \frac{X_k}{X^TX} \f]
- *  "ej" es el error de la salida de la j-essima neurona.
- *  "func" es la función de activación da neurona.
- *  \param[in,out] NVector El vector de neuronas a actualizar los pesos.
- *  \param[in] Alpha Factor de aprendizaje de los pesos.
- *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL). 
- *  \ingroup PdsNVectorGroup
- */
-int pds_nvector_update_weight_normalized(PdsNVector *NVector, PdsNnReal Alpha);
-
-
 //@}
 
 /** @name pds_nvector_<printf>
@@ -402,7 +365,7 @@ int pds_nvector_update_weight_normalized(PdsNVector *NVector, PdsNnReal Alpha);
 
 
 /** \fn int pds_nvector_fprintf(const PdsNVector *NVector, FILE *fd)
- *  \brief Guarda en un archivo de texto los pesos \f$W_{kj}\f$ y los valores \f$\{U_j,\sigma_j\}\f$.
+ *  \brief Guarda en un archivo de texto los pesos \f$W_{kj}\f$ y los valores \f$\{U_j\}\f$.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  \param[in] NVector El vector de neuronas a leer.
  *  \param[in,out] fd Manejador del fichero a escribir.
@@ -413,7 +376,7 @@ int pds_nvector_fprintf(const PdsNVector *NVector, FILE *fd);
 
 
 /** \fn int pds_nvector_fscanf(PdsNVector *NVector, FILE *fd)
- *  \brief Lee de un archivo de texto los pesos \f$W_{kj}\f$ y los valores \f$\{U_j,\sigma_j\}\f$.
+ *  \brief Lee de un archivo de texto los pesos \f$W_{kj}\f$ y los valores \f$\{U_j\}\f$.
  *  Ocupando una linea cada uno, y separando los elementos por un TAB.
  *  \param[out] NVector El vector a escribir.
  *  \param[in,out] fd Manejador del fichero a leer.
@@ -424,7 +387,7 @@ int pds_nvector_fscanf(PdsNVector *NVector, FILE *fd);
 
 
 /** \fn int pds_nvector_fwrite(const PdsNVector *NVector, FILE *fd)
- *  \brief Guarda en un archivo de texto binario los pesos \f$W_{kj}\f$ y los valores \f$\{U_j,\sigma_j\}\f$.
+ *  \brief Guarda en un archivo de texto binario los pesos \f$W_{kj}\f$ y los valores \f$\{U_j\}\f$.
  *  \param[in] NVector El vector de neuronas a leer.
  *  \param[in,out] fd Manejador del fichero binario a escribir.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL o fd==NULL). 
@@ -434,7 +397,7 @@ int pds_nvector_fwrite(const PdsNVector *NVector, FILE *fd);
 
 
 /** \fn int pds_nvector_fread(PdsNVector *NVector, FILE *fd)
- *  \brief Lee de un archivo de texto binario los pesos \f$W_{kj}\f$ y los valores \f$\{U_j,\sigma_j\}\f$.
+ *  \brief Lee de un archivo de texto binario los pesos \f$W_{kj}\f$ y los valores \f$\{U_j\}\f$.
  *  \param[out] NVector El vector a escribir.
  *  \param[in,out] fd Manejador del fichero binario a leer.
  *  \return TRUE si todo fue bien o FALSE si no (ej: NVector==NULL o fd==NULL). 
